import json
import os
import sys
import argparse

from jupyter_client.kernelspec import KernelSpecManager
from IPython.utils.tempdir import TemporaryDirectory

kernel_json = {
 "argv":[sys.executable,"-m","lithium_kernel", "-f", "{connection_file}"],
 "display_name":"Lithium",
 "language":"lithium"
}

# todo: download & register execution host

def install_my_kernel_spec(user=True, prefix=None):
    with TemporaryDirectory() as td:
        os.chmod(td, 0o755)
        with open(os.path.join(td, 'kernel.json'), 'w') as f:
            json.dump(kernel_json, f, sort_keys=True)

        print('Installing IPython kernel spec')
        KernelSpecManager().install_kernel_spec(td, 'lithium', user=user, prefix=prefix)

def main(argv=None):
    parser = argparse.ArgumentParser(description='Install KernelSpec for Lithium Kernel')
    prefix_locations = parser.add_mutually_exclusive_group()

    prefix_locations.add_argument(
        '--user',
        help='Install KernelSpec in user home',
        action='store_true'
    )
    prefix_locations.add_argument(
        '--sys-prefix',
        help='Install KernelSpec in sys.prefix. Useful in conda / virtualenv',
        action='store_true',
        dest='sys_prefix'
    )
    prefix_locations.add_argument(
        '--prefix',
        help='Install KernelSpec in this prefix',
        default=None
    )

    args = parser.parse_args(argv)

    user = False
    prefix = None
    if args.sys_prefix:
        prefix = sys.prefix
    elif args.prefix:
        prefix = args.prefix
    elif args.user:
        user = True

    install_my_kernel_spec(user=user, prefix=prefix)

if __name__ == '__main__':
    main()



from ipykernel.kernelbase import Kernel
import json
import requests
import logging

__version__ = '0.2.0'

api_base = 'http://localhost:61494/'
api_run = 'code/run'
api_defs = 'api/definitions'

def load_definitions():
    return json.loads(requests.get(api_base + api_defs, verify=False).text)

class LithiumKernel(Kernel):
    implementation = 'Lithium'
    implementation_version = __version__
    language = 'lithium-lisp'
    language_version = '2022.8.31.1105'
    language_info = {
        'name': 'Lithium LISP',
        'mimetype': 'application/x-lisp',
        'file_extension': '.lsp',
    }
    banner = "Lithium LISP - programming language for ReLife Platform"

    def run_code(self, module, code):
        try:
            js = {'module':module, 'code':code}
            res = json.loads(requests.post(api_base + api_run, json=js, verify=False).text)

            if res['download'] is not None:
                ht = requests.get(res['download'], verify=False).text
                return self.send_html(ht, res['redirect'])
            else:
                return self.send_plain(res['result'])
        except Exception as e:
            logging.error(e)

    def send_html(self, h, l):
        self.send_response(self.iopub_socket, 'display_data', {
            'data': {
                'text/html': '<a href="{}">View via host</a><br/>'.format(l) + h,
                'text/plain': l
            },
            'metadata': {}
        })

    def send_plain(self, r):
        self.send_response(self.iopub_socket, 'execute_result', {
            'data': {
                'text/plain': str(r)
            },
            'execution_count': self.execution_count,
            'metadata':{}
        })

    def do_execute(self, code, silent, store_history=True, user_expressions=None, allow_stdin=False):
        if not silent:
            self.run_code('t', code)

        return {
            'status': 'ok',
            'execution_count': self.execution_count,
            'payload': [],
            'user_expressions': {}
        }